package com.ibm.sysmgt.raidmgr.cim.provider.event;

import com.ibm.sysmgt.raidmgr.util.*;
import com.ibm.sysmgt.raidmgr.dataproc.config.*;
import com.ibm.sysmgt.raidmgr.dataproc.config.lsi.*;
import com.ibm.sysmgt.raidmgr.cim.provider.*;
import com.ibm.sysmgt.raidmgr.cim.provider.instance.*;

import com.sun.wbem.cim.*;
import com.sun.wbem.provider.CIMProvider;
import com.sun.wbem.provider20.Authorizable;
import com.sun.wbem.provider20.EventProvider;
import com.sun.wbem.query.SelectExp;

import java.util.*;
import java.io.*;
import java.rmi.RemoteException;

/** Processes and sends events to CIM consumers via the UMS cimwmi proxy provider.
  * Events are recieved from EventPollingThread which polls the ServeRAID
  * Director agent for events and notifies this object when events are recieved
  * via the ProcessAlert method.
  */
public class RAIDEventProvider extends RAIDProvider implements AlertListener, EventProvider
{
   protected static PrintWriter eventLog = setupLog("RAIDEventProvider.log");

   EventPollingThread pollingThread;
   Hashtable eventTable;

   public final static int NORMAL_EVENT = 0;
   public final static int WARNING_EVENT = 1;
   public final static int CRITICAL_EVENT = 2;
   final static String RAID_EVENT = "IBMPSG_StorageRAIDEvent";
   final static String RAID_HEALTH_EVENT = "IBMPSG_StorageRAIDHealthEvent";

   public RAIDEventProvider()
   {            
      super("RAIDEventProvider:");
      eventLog.println(logHeader + "RAIDEventProvider()");
      initializeEventTable();
   }

   public void authorizeFilter(SelectExp filter, String eventType, CIMObjectPath classPath, String owner) throws CIMException
   {
      eventLog.println(logHeader + "authorizeFilter()");
   }

   /** Satisfies the EventProvider interface */
   public void activateFilter(SelectExp filter, String eventType, CIMObjectPath classPath, boolean firstActivation) throws CIMException
   {
      if (eventType.equals(RAID_EVENT) || eventType.equals(RAID_HEALTH_EVENT)) 
      {                                    
         if (firstActivation)
         {         
            pollingThread = new EventPollingThread(this, eventLog);
            // Ask the agent to send the initial set of events
            //pollingThread.getInitialEvents();
            pollingThread.start();
         }
      }
      else
      {
         eventLog.println(logHeader + "activateFilter() - Does not support eventType " + eventType);
         throw new CIMException(CIMException.CIM_ERR_NOT_SUPPORTED);
      }
   }

   /** Satisfies the EventProvider interface */
   public void deActivateFilter(SelectExp filter, String eventType, CIMObjectPath classPath, boolean lastActivation) throws CIMException
   {
      eventLog.println(logHeader + "deActivateFilter()");
      pollingThread.endThread();
   }

   /** Satisfies the EventProvider interface */
   public boolean mustPoll(SelectExp filter, String eventType, CIMObjectPath classPath) throws CIMException
   {
      // Not sure what this does
      eventLog.println(logHeader + "mustPoll()");
      return false;
   }

   /** Sends the event recieved in ProcessAlert to the cimwmi proxy provider
     * who in turn forwards the event to the various CIM event consumers.
     */
   void sendEvent(RaidEvent event, String eventType)
   {
      eventLog.println("sendEvent() eventType = " + eventType);

      // Required by WMI
      CIMNameSpace ns = new CIMNameSpace(".", "root\\ibmsd");
      CIMObjectPath cop = new CIMObjectPath(eventType);
      try
      {
          CIMInstance indication = cimom.getClass(cop, false).newInstance();
          cimom.deliverEvent("root\\ibmsd", mapEvent(indication, event));
      }
      catch(Exception e)
      {
          e.printStackTrace(eventLog);
      }
   }

   /** Translates a RaidEvent into a CIM event */
   CIMInstance mapEvent(CIMInstance indication, RaidEvent event)
   {            
      int eventSeverity = RAIDEventProvider.NORMAL_EVENT;
      String targetObjectPath = "";

      switch(event.getEventType())
      {
      case Constants.WARNING_EVENT:
         eventSeverity = RAIDEventProvider.WARNING_EVENT;         
         targetObjectPath = getTargetObjectPath(event);
         break;
      case Constants.FATAL_EVENT:
         eventSeverity = RAIDEventProvider.CRITICAL_EVENT;
         targetObjectPath = getTargetObjectPath(event);
         break;
      case Constants.INFORMATION_EVENT:
         eventSeverity = RAIDEventProvider.NORMAL_EVENT;
         targetObjectPath = getTargetObjectPath(event);
         break;
      case Constants.PROGRESS_EVENT:
         eventSeverity = RAIDEventProvider.NORMAL_EVENT;
         targetObjectPath = getTargetObjectPath(event);
         break;
      case Constants.OVERALL_STATUS_CRITICAL_EVENT:          
         eventSeverity = RAIDEventProvider.WARNING_EVENT;
         targetObjectPath = ServeRaidOverallStatusProvider.CIM_CREATION_CLASS_NAME + ".SettingID=\"" + ServeRaidOverallStatusProvider.SETTING_ID + "\"";
         break;
      case Constants.OVERALL_STATUS_FATAL_EVENT:
         eventSeverity = RAIDEventProvider.CRITICAL_EVENT;
         targetObjectPath = ServeRaidOverallStatusProvider.CIM_CREATION_CLASS_NAME + ".SettingID=\"" + ServeRaidOverallStatusProvider.SETTING_ID + "\"";
         break;
      case Constants.OVERALL_STATUS_NORMAL_EVENT:  
         eventSeverity = RAIDEventProvider.NORMAL_EVENT;
         targetObjectPath = ServeRaidOverallStatusProvider.CIM_CREATION_CLASS_NAME + ".SettingID=\"" + ServeRaidOverallStatusProvider.SETTING_ID + "\"";
         break;
      default:
      };      

      indication.setProperty("Identifier", new CIMValue("ServeRAID Subsystem"));
      indication.setProperty("SourceObjectPath", new CIMValue(""));

      // For now the target will always be the controller since there is currently no easy way
      // to associate an event and object.
      indication.setProperty("TargetObjectPath", new CIMValue(targetObjectPath));
      indication.setProperty("Description", new CIMValue("ServeRAID:  " + event.getEventString()));
      indication.setProperty("Severity", new CIMValue(new UnsignedInt16(eventSeverity)));
      indication.setProperty("TimeStamp", new CIMValue(new CIMDateTime()));            

      indication.setProperty("ServerName", new CIMValue(event.getServerName()));
      indication.setProperty("ErrorCode", new CIMValue(new UnsignedInt16(event.getErrorCode())));
      indication.setProperty("AdapterID", new CIMValue(new UnsignedInt16(event.getAdapterNumber())));
      indication.setProperty("ChannelID", new CIMValue(new UnsignedInt16(event.getChannelID())));
      indication.setProperty("SCSIID", new CIMValue(new UnsignedInt16(event.getSCSIID())));
      indication.setProperty("LogicalDriveID", new CIMValue(new UnsignedInt16(event.getLogicalDriveID())));
      indication.setProperty("ArrayID", new CIMValue(event.getArrayID()));
      indication.setProperty("FRU", new CIMValue(event.getFRU()));
      indication.setProperty("FanID", new CIMValue(new UnsignedInt16(event.getFanID())));
      indication.setProperty("PowerSupplyID", new CIMValue(new UnsignedInt16(event.getPowerSupplyID())));

      return indication;
   }

   /**
     * This method associates an appropiate CIM object with an event.  For example a defunct drive event is associated
     * with a disk drive, and a critical logical drive event is associated with the appropiate logical drive.
     * @param evt the RaidEvent object received from the ServeRAID Manager agent. 
     # @return String representing the associated object's ObjectPath
     */ 
   String getTargetObjectPath(RaidEvent evt)
   {
      int eventCode = evt.getEventCode();

      String systemCreationClassName = com.ibm.sysmgt.raidmgr.cim.provider.CIMUtil.getSystemCreationClassName();
      String systemName = JCRMNet.getHostname();
      String creationClassName = null;
      String deviceID = null;      

      // Logical Drive Events
      if ((eventCode > 300) && (eventCode < 321))
      {
         creationClassName = LogicalDrive.CIM_CREATION_CLASS_NAME;
         deviceID = new String("IBM:ServeRAID:ControllerID:" + evt.getAdapterNumber() + ":LogicalDriveID:" + evt.getLogicalDriveID() + ":LogicalDrive");
      }
      // Spanned Array Events
      else if ((eventCode > 321) && (eventCode < 331))
      {
         creationClassName = SpannedArray.CIM_CREATION_CLASS_NAME;
         String collectionID = new String("IBM:ServeRAID:ControllerID:" + evt.getAdapterNumber() + ":SpannedArrayID:" + evt.getArrayID() + ":SpannedArray");
         return creationClassName + ".CreationClassName=\"" + creationClassName + "\"" + ",SystemCreationClassName=\"" + systemCreationClassName + "\"" + ",SystemName=\"" + systemName + "\"" + ",CollectionID=\"" + collectionID +  "\"";
      }
      // More Logical Drive Events
      else if ((eventCode > 330) && (eventCode < 400))
      {
         creationClassName = LogicalDrive.CIM_CREATION_CLASS_NAME;
         deviceID = new String("IBM:ServeRAID:ControllerID:" + evt.getAdapterNumber() + ":LogicalDriveID:" + evt.getLogicalDriveID() + ":LogicalDrive");
      }
      // Disk Drive Events
      else if ((eventCode > 400) && (eventCode < 500))
      {
         creationClassName = HardDrive.CIM_CREATION_CLASS_NAME;
         deviceID = new String("IBM:ServeRAID:ControllerID:" + evt.getAdapterNumber() + ":ChannelID:" + evt.getChannelID() + ":DiskDriveID:" + evt.getSCSIID() + ":DiskDrive");
      }
      // Enclosure Events
      else if ((eventCode > 500) && (eventCode < 600))
      {
         creationClassName = EnclosureDevice.CIM_CREATION_CLASS_NAME;
         deviceID = new String("IBM:ServeRAID:ControllerID:" + evt.getAdapterNumber() + ":ChannelID:" + evt.getChannelID() + ":EnclosureID:" + evt.getSCSIID() + ":Enclosure");
      }
      else
         // Default to the Controller
      {
         creationClassName = Adapter.CIM_CREATION_CLASS_NAME;
         Adapter adapter = null;

         try
         {
            adapter = getRaidSystem().getAdapter(Adapter.adjustedIDToAdapterID(evt.getAdapterNumber()));
         }
         catch (RemoteException e)
         {     
            return Adapter.CIM_CREATION_CLASS_NAME + ".CreationClassName=\"" + Adapter.CIM_CREATION_CLASS_NAME + "\"" + ",SystemCreationClassName=\"" + systemCreationClassName + "\"" + ",SystemName=\"" + systemName + "\"" + ",DeviceID=\"" + (evt.getAdapterNumber() - 1)+  "\"";
         }         

         if (adapter instanceof ServeRaidAdapter)
            creationClassName = ServeRaidAdapter.CIM_CREATION_CLASS_NAME;
         else if (adapter instanceof LSIAdapter)
            creationClassName = LSIAdapter.CIM_CREATION_CLASS_NAME;

         deviceID = new String("IBM:ServeRAID:ControllerID:" + adapter.getDisplayID());
      }

      return creationClassName + ".CreationClassName=\"" + creationClassName + "\"" + ",SystemCreationClassName=\"" + systemCreationClassName + "\"" + ",SystemName=\"" + systemName + "\"" + ",DeviceID=\"" + deviceID +  "\"";
   }

   /**
     * This method processes the alerts received from the ServeRAID Manager agent. 
     * @param evt the RaidEvent object received from the ServeRAID Manager agent. 
     */ 
   public void processAlert(RaidEvent evt)
   {    
      eventLog.println();
      eventLog.println(evt);
      if (evt.getEventType() == Constants.OVERALL_STATUS_NORMAL_EVENT   ||
          evt.getEventType() == Constants.OVERALL_STATUS_CRITICAL_EVENT ||
          evt.getEventType() == Constants.OVERALL_STATUS_FATAL_EVENT)
      {                                             
         sendEvent(evt, RAID_HEALTH_EVENT);         
      }
      else
      {
         String eventType = (String)eventTable.get(new Integer(evt.getEventCode()));
         sendEvent(evt, eventType == null ? RAID_EVENT : eventType);
      }
   }

   private void initializeEventTable()
   {      
      eventTable = new Hashtable(80, (float).75);

      eventTable.put(new Integer(Constants.NO_CONTROLLERS), "IBMPSG_ServeRAIDNoControllers");
      eventTable.put(new Integer(Constants.CONTROLLER_FAIL), "IBMPSG_ServeRAIDControllerFail");
      eventTable.put(new Integer(Constants.DEAD_BATTERY), "IBMPSG_ServeRAIDDeadBattery");
      eventTable.put(new Integer(Constants.DEAD_BATTERY_CACHE), "IBMPSG_ServeRAIDDeadBatteryCache");
      eventTable.put(new Integer(Constants.POLLING_FAIL), "IBMPSG_ServeRAIDPollingFail");
      eventTable.put(new Integer(Constants.CONFIG_FAIL), "IBMPSG_ServeRAIDConfigFail");
      eventTable.put(new Integer(Constants.CONTROLLER_ADDED), "IBMPSG_ServeRAIDControllerAdded");
      eventTable.put(new Integer(Constants.CONTROLLER_REPLACED), "IBMPSG_ServeRAIDControllerReplaced");
      eventTable.put(new Integer(Constants.CONTROLLER_FAILOVER), "IBMPSG_ServeRAIDControllerFailover");
      eventTable.put(new Integer(Constants.MISMATCHED_VERSIONS), "IBMPSG_ServeRAIDControllerMismatchedVersions");

      eventTable.put(new Integer(Constants.LOGICAL_DRIVE_CRITICAL), "IBMPSG_ServeRAIDLogicalDriveCritical");
      eventTable.put(new Integer(Constants.LOGICAL_DRIVE_BLOCKED), "IBMPSG_ServeRAIDLogicalDriveBlocked");
      eventTable.put(new Integer(Constants.LOGICAL_DRIVE_OFFLINE), "IBMPSG_ServeRAIDLogicalDriveOffLine");
      eventTable.put(new Integer(Constants.REBUILD_DETECTED), "IBMPSG_ServeRAIDRebuildDetected");
      eventTable.put(new Integer(Constants.REBUILD_COMPLETE), "IBMPSG_ServeRAIDRebuildComplete");
      eventTable.put(new Integer(Constants.REBUILD_FAIL), "IBMPSG_ServeRAIDRebuildFail");
      eventTable.put(new Integer(Constants.SYNC_DETECTED), "IBMPSG_ServeRAIDSyncDetected");
      eventTable.put(new Integer(Constants.SYNC_COMPLETE), "IBMPSG_ServeRAIDSyncComplete");
      eventTable.put(new Integer(Constants.SYNC_FAIL), "IBMPSG_ServeRAIDSyncFail");
      eventTable.put(new Integer(Constants.MIGRATION_DETECTED), "IBMPSG_ServeRAIDMigrationDetected");
      eventTable.put(new Integer(Constants.MIGRATION_COMPLETE), "IBMPSG_ServeRAIDMigrationComplete");
      eventTable.put(new Integer(Constants.MIGRATION_FAIL), "IBMPSG_ServeRAIDMigrationFail");
      eventTable.put(new Integer(Constants.COMPRESSION_DETECTED), "IBMPSG_ServeRAIDCompressionDetected");
      eventTable.put(new Integer(Constants.COMPRESSION_COMPLETE), "IBMPSG_ServeRAIDCompressionComplete");
      eventTable.put(new Integer(Constants.COMPRESSION_FAIL), "IBMPSG_ServeRAIDCompressionFail");
      eventTable.put(new Integer(Constants.DECOMPRESSION_DETECTED), "IBMPSG_ServeRAIDDecompressionDetected");
      eventTable.put(new Integer(Constants.DECOMPRESSION_COMPLETE), "IBMPSG_ServeRAIDDecompressionComplete");
      eventTable.put(new Integer(Constants.DECOMPRESSION_FAIL), "IBMPSG_ServeRAIDDecompressionFail");
      eventTable.put(new Integer(Constants.FLASHCOPY_DETECTED), "IBMPSG_ServeRAIDFlashCopyDetected");
      eventTable.put(new Integer(Constants.FLASHCOPY_COMPLETE), "IBMPSG_ServeRAIDFlashCopyComplete");
      eventTable.put(new Integer(Constants.FLASHCOPY_FAIL), "IBMPSG_ServeRAIDFlashCopyFail");
      eventTable.put(new Integer(Constants.ARRAY_REBUILD_DETECTED), "IBMPSG_ServeRAIDArrayRebuildDetected");
      eventTable.put(new Integer(Constants.ARRAY_REBUILD_COMPLETE), "IBMPSG_ServeRAIDArrayRebuildComplete");
      eventTable.put(new Integer(Constants.ARRAY_REBUILD_FAIL), "IBMPSG_ServeRAIDArrayRebuildFail");
      eventTable.put(new Integer(Constants.ARRAY_SYNC_DETECTED), "IBMPSG_ServeRAIDArraySyncDetected");
      eventTable.put(new Integer(Constants.ARRAY_SYNC_COMPLETE), "IBMPSG_ServeRAIDArraySyncComplete");
      eventTable.put(new Integer(Constants.ARRAY_SYNC_FAIL), "IBMPSG_ServeRAIDArraySyncFail");
      eventTable.put(new Integer(Constants.ARRAY_FLASHCOPY_DETECTED), "IBMPSG_ServeRAIDArrayFlashCopyDetected");
      eventTable.put(new Integer(Constants.ARRAY_FLASHCOPY_COMPLETE), "IBMPSG_ServeRAIDArrayFlashCopyComplete");
      eventTable.put(new Integer(Constants.ARRAY_FLASHCOPY_FAIL), "IBMPSG_ServeRAIDArrayFlashCopyFail");
      eventTable.put(new Integer(Constants.LOGICAL_DRIVE_UNBLOCKED), "IBMPSG_ServeRAIDLogicalDriveUnblocked");

      eventTable.put(new Integer(Constants.DEFUNCT_DRIVE), "IBMPSG_ServeRAIDDefunctDrive");
      eventTable.put(new Integer(Constants.PFA_DRIVE), "IBMPSG_ServeRAIDPFADrive");
      eventTable.put(new Integer(Constants.DEFUNCT_REPLACED), "IBMPSG_ServeRAIDDefunctReplaced");
      eventTable.put(new Integer(Constants.DEFUNCT_DRIVE_FRU), "IBMPSG_ServeRAIDDefunctDriveFRU");
      eventTable.put(new Integer(Constants.PFA_DRIVE_FRU), "IBMPSG_ServeRAIDPFADriveFRU");
      eventTable.put(new Integer(Constants.UNSUPPORTED_DRIVE), "IBMPSG_ServeRAIDUnsupportedDrive");      

      eventTable.put(new Integer(Constants.ENCLOSURE_OK), "IBMPSG_ServeRAIDEnclosureOK");
      eventTable.put(new Integer(Constants.ENCLOSURE_FAIL), "IBMPSG_ServeRAIDEnclosureFail");
      eventTable.put(new Integer(Constants.FAN_OK), "IBMPSG_ServeRAIDEnclosureFanOk");
      eventTable.put(new Integer(Constants.FAN_FAIL), "IBMPSG_ServeRAIDEnclosureFanFail");
      eventTable.put(new Integer(Constants.FAN_INSTALLED), "IBMPSG_ServeRAIDEnclosureFanInstalled");
      eventTable.put(new Integer(Constants.FAN_REMOVED), "IBMPSG_ServeRAIDEnclosureFanRemoved");
      eventTable.put(new Integer(Constants.TEMP_OK), "IBMPSG_ServeRAIDEnclosureTempOk");
      eventTable.put(new Integer(Constants.TEMP_FAIL), "IBMPSG_ServeRAIDEnclosureTempFail");
      eventTable.put(new Integer(Constants.POWER_SUPPLY_OK), "IBMPSG_ServeRAIDEnclosurePowerSupplyOk");
      eventTable.put(new Integer(Constants.POWER_SUPPLY_FAIL), "IBMPSG_ServeRAIDEnclosurePowerSupplyFail");
      eventTable.put(new Integer(Constants.POWER_SUPPLY_INSTALLED), "IBMPSG_ServeRAIDEnclosurePowerSupplyInstalled");
      eventTable.put(new Integer(Constants.POWER_SUPPLY_REMOVED), "IBMPSG_ServeRAIDEnclosurePowerSupplyRemoved");

      eventTable.put(new Integer(Constants.TEST_TRAP), "IBMPSG_ServeRAIDTestEvent");      
   }

   public static void main(String[] args)
   {
      RAIDEventProvider provider = new RAIDEventProvider();

      try
      {
         provider.activateFilter(null, RAID_EVENT, null, true);
      }
      catch (CIMException e)
      {
         e.printStackTrace();
         return;
      }
   }
}
