package com.ibm.sysmgt.raidmgr.cim.provider.instance;

import com.ibm.sysmgt.raidmgr.util.*;
import com.ibm.sysmgt.raidmgr.dataproc.*;
import com.ibm.sysmgt.raidmgr.dataproc.config.*;
import com.ibm.sysmgt.raidmgr.dataproc.util.*;
import com.ibm.sysmgt.raidmgr.cim.provider.*;

import com.sun.wbem.cim.*;
import com.sun.wbem.provider.CIMProvider;
import com.sun.wbem.provider20.Authorizable;
import com.sun.wbem.provider20.InstanceProvider;
import com.sun.wbem.client.ProviderCIMOMHandle;
import com.sun.wbem.query.SelectExp;

import java.util.*;
import java.io.*;
import java.rmi.RemoteException;

public class EnclosureProvider extends RAIDInstanceProviderBase
{
   public EnclosureProvider()
   {
      super("EnclosureProvider:");
      instanceLog.println(logHeader + "EnclosureProvider()");
   }

   public Vector enumInstances(CIMObjectPath op, boolean deep, CIMClass cc) throws CIMException
   {
      instanceLog.println(logHeader + "enumInstances(" + op.getObjectName() + ") Names Only");

      SetInteractiveWSAD();

      Vector cops = new Vector();
      Vector instances = null;

      RaidSystem system = null;
      try
      {         
         system = (RaidSystem)getRaidSystem();
      }
      catch (RemoteException e)
      {
         instanceLog.println(logHeader + "Error accessing the ServeRAID Manager agent. Exception: " + e);
         RestoreInteractiveWSAD();
         throw new CIMException(CIMException.CIM_ERR_FAILED);
      }

      if (op.getObjectName().equalsIgnoreCase(EnclosureDevice.CIM_CREATION_CLASS_NAME))
         instances = enumEnclosures(system, cc);
      else if (op.getObjectName().equalsIgnoreCase(EnclosureDevice.CIM_FAN_CLASS_NAME))
         instances = enumFans(system, cc);
      else if (op.getObjectName().equalsIgnoreCase(EnclosureDevice.CIM_POWER_SUPPLY_CLASS_NAME))
         instances = enumPowerSupplies(system, cc);
      else if (op.getObjectName().equalsIgnoreCase(EnclosureDevice.CIM_TEMPERATURE_CLASS_NAME))
         instances = enumTemperatures(system, cc);
      else if (op.getObjectName().equalsIgnoreCase(EnclosureDevice.CIM_SAFTE_CHIP_CLASS_NAME))
         instances = enumSafteChips(system, cc);
      else
      {
         instanceLog.println(logHeader + "Does not provide names of class " + op.getObjectName());
         RestoreInteractiveWSAD();
         throw (new CIMException(CIMException.CIM_ERR_NOT_SUPPORTED));
      }

      for (Enumeration e = instances.elements(); e.hasMoreElements();)
      {
         CIMInstance ci = (CIMInstance)e.nextElement();
         CIMObjectPath cop = new CIMObjectPath(cc.getName(), op.getNameSpace());
         cop.setKeys(ci.getKeyValuePairs());
         cops.addElement(cop);
      }

      RestoreInteractiveWSAD();
      return cops;
   }

   public Vector enumInstances(CIMObjectPath op, boolean deep, CIMClass cc, boolean localOnly) throws CIMException
   {
      instanceLog.println(logHeader + "enumInstances(" + op.getObjectName() + ")");

      SetInteractiveWSAD();

      if (isConnectedToAgent() == false)
      {
         instanceLog.println(logHeader + "Cannot enumerate classes, not connected to agent.");
         RestoreInteractiveWSAD();
         throw new CIMException(CIMException.CIM_ERR_FAILED);
      }

      RaidSystem system = null;
      try
      {         
         system = (RaidSystem)getRaidSystem();
      }
      catch (RemoteException e)
      {
         instanceLog.println(logHeader + "Error accessing the ServeRAID Manager agent. Exception: " + e);
         RestoreInteractiveWSAD();
         throw new CIMException(CIMException.CIM_ERR_FAILED);
      }

      if (op.getObjectName().equalsIgnoreCase(EnclosureDevice.CIM_CREATION_CLASS_NAME))
      {
         RestoreInteractiveWSAD();
         return enumEnclosures(system, cc);
      }
      if (op.getObjectName().equalsIgnoreCase(EnclosureDevice.CIM_FAN_CLASS_NAME))
      {
         RestoreInteractiveWSAD();      
         return enumFans(system, cc);
      }
      if (op.getObjectName().equalsIgnoreCase(EnclosureDevice.CIM_POWER_SUPPLY_CLASS_NAME))
      {
         RestoreInteractiveWSAD();      
         return enumPowerSupplies(system, cc);
      }
      if (op.getObjectName().equalsIgnoreCase(EnclosureDevice.CIM_TEMPERATURE_CLASS_NAME))
      {
         RestoreInteractiveWSAD();
         return enumTemperatures(system, cc);
      }
      if (op.getObjectName().equalsIgnoreCase(EnclosureDevice.CIM_SAFTE_CHIP_CLASS_NAME))
      {
         RestoreInteractiveWSAD();
         return enumSafteChips(system, cc);
      }
      else
      {
         instanceLog.println(logHeader + "Does not provide instances of class " + op.getObjectName());
         RestoreInteractiveWSAD();
         throw (new CIMException(CIMException.CIM_ERR_NOT_SUPPORTED));
      }
   }

   public CIMInstance getInstance(CIMObjectPath op, CIMClass cc, boolean localOnly) throws CIMException
   {
      instanceLog.println(logHeader + "getInstance(" + op.getObjectName() + ")");

      SetInteractiveWSAD();

      if (isConnectedToAgent() == false)
      {
         instanceLog.println(logHeader + "Can't get class of type "  + op.getObjectName() + ", not connected to agent.");
         RestoreInteractiveWSAD();
         throw new CIMException(CIMException.CIM_ERR_FAILED);
      }

      RaidSystem system = null;
      try
      {
         Object[] args = null;
         system = getRaidSystem();
      }
      catch (RemoteException e)
      {
         instanceLog.println(logHeader + "Error accessing the ServeRAID Manager agent. Exception: " + e);
         RestoreInteractiveWSAD();
         throw new CIMException(CIMException.CIM_ERR_FAILED);
      }

      if (op.getObjectName().equalsIgnoreCase(EnclosureDevice.CIM_CREATION_CLASS_NAME))
      {
         RestoreInteractiveWSAD();
         return getEnclosure(system, cc, op);
      }
      if (op.getObjectName().equalsIgnoreCase(EnclosureDevice.CIM_FAN_CLASS_NAME))
      {
         RestoreInteractiveWSAD();
         return getFan(system, cc, op);
      }
      if (op.getObjectName().equalsIgnoreCase(EnclosureDevice.CIM_POWER_SUPPLY_CLASS_NAME))
      {
         RestoreInteractiveWSAD();
         return getPowerSupply(system, cc, op);
      }
      if (op.getObjectName().equalsIgnoreCase(EnclosureDevice.CIM_TEMPERATURE_CLASS_NAME))
      {
         RestoreInteractiveWSAD(); 
         return getTemperature(system, cc, op);
      }
      if (op.getObjectName().equalsIgnoreCase(EnclosureDevice.CIM_SAFTE_CHIP_CLASS_NAME))
      {
         RestoreInteractiveWSAD();
         return getSafteChip(system, cc, op);
      }
      else
      {
         instanceLog.println(logHeader + "Does not provide instances of class " + op.getObjectName());
         RestoreInteractiveWSAD();
         throw (new CIMException(CIMException.CIM_ERR_NOT_SUPPORTED));
      }      
   }

   // **********************
   // Enclosures
   // **********************
   public Vector enumEnclosures(RaidSystem system, CIMClass cc) throws CIMException
   {
      Vector instances = new Vector();

      for (Enumeration e = system.getPhysicalDeviceCollection(new PhysicalDeviceTypeFilter(Constants.ENCLOSURE)).elements(); e.hasMoreElements();)
      {
         EnclosureDevice enclosure = (EnclosureDevice)e.nextElement();
         instances.addElement(enclosure.createCIMInstance(cc.newInstance()));
      }

      return instances;
   }   

   public CIMInstance getEnclosure(RaidSystem system, CIMClass cc, CIMObjectPath op) throws CIMException
   {
      try
      {
         String deviceID = (String)getKeyValue(op, "DeviceID");
         int enclosureID = Integer.parseInt(getToken(deviceID, ":", "EnclosureID"));
         int channelID = Channel.displayIDToChannelID(getToken(deviceID, ":", "ChannelID"));
         int controllerID = Adapter.displayIDToAdapterID(getToken(deviceID, ":", "ControllerID"));

         Adapter adapter = (Adapter)system.getAdapter(controllerID);
         
         PhysicalDevice_DeviceIDFilter filter = new PhysicalDevice_DeviceIDFilter(new DeviceID(controllerID, channelID, enclosureID));
         Vector devices = adapter.getPhysicalDeviceCollection(filter);
         EnclosureDevice enclosure = (EnclosureDevice) devices.elementAt(0);
         return enclosure.createCIMInstance(cc.newInstance());
      }
      catch (ClassCastException e)
      {
         e.printStackTrace(log);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }
      catch (NumberFormatException e)
      {
         e.printStackTrace(log);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }      
      catch (NullPointerException e)
      {
         e.printStackTrace(log);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }
      catch (ArrayIndexOutOfBoundsException e)
      {
         e.printStackTrace(log);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }
   }

   // **********************
   // Enclosure Fans
   // **********************
   public Vector enumFans(RaidSystem system, CIMClass cc) throws CIMException
   {
      Vector instances = new Vector();

      for (Enumeration e = system.getPhysicalDeviceCollection(new PhysicalDeviceTypeFilter(Constants.ENCLOSURE)).elements(); e.hasMoreElements();)
      {
         EnclosureDevice enclosure = (EnclosureDevice)e.nextElement();
         for (int i = 0; i < enclosure.getFanCount(); i++)
            instances.addElement(enclosure.createCIMFanInstance(cc.newInstance(), i));         
      }

      return instances;
   }   

   public CIMInstance getFan(RaidSystem system, CIMClass cc, CIMObjectPath op) throws CIMException
   {
      try
      {
         String deviceID = (String)getKeyValue(op, "DeviceID");
         int fanID = Integer.parseInt(getToken(deviceID, ":", "FanID")) - 1;
         int enclosureID = Integer.parseInt(getToken(deviceID, ":", "EnclosureID"));
         int channelID = Channel.displayIDToChannelID(getToken(deviceID, ":", "ChannelID"));
         int controllerID = Adapter.displayIDToAdapterID(getToken(deviceID, ":", "ControllerID"));

         Adapter adapter = (Adapter)system.getAdapter(controllerID);
         
         PhysicalDevice_DeviceIDFilter filter = new PhysicalDevice_DeviceIDFilter(new DeviceID(controllerID, channelID, enclosureID));
         Vector devices = adapter.getPhysicalDeviceCollection(filter);
         EnclosureDevice enclosure = (EnclosureDevice) devices.elementAt(0);
         return enclosure.createCIMFanInstance(cc.newInstance(),fanID);
      }
      catch (ClassCastException e)
      {
         e.printStackTrace(log);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }
      catch (NumberFormatException e)
      {
         e.printStackTrace(log);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }      
      catch (NullPointerException e)
      {
         e.printStackTrace(log);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }
      catch (ArrayIndexOutOfBoundsException e)
      {
         e.printStackTrace(log);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }
   }

   // **********************
   // Enclosure Power Supplies
   // **********************
   public Vector enumPowerSupplies(RaidSystem system, CIMClass cc) throws CIMException
   {
      Vector instances = new Vector();

      for (Enumeration e = system.getPhysicalDeviceCollection(new PhysicalDeviceTypeFilter(Constants.ENCLOSURE)).elements(); e.hasMoreElements();)
      {
         EnclosureDevice enclosure = (EnclosureDevice)e.nextElement();
         for (int i = 0; i < enclosure.getPowerSupplyCount(); i++)
            instances.addElement(enclosure.createCIMPowerSupplyInstance(cc.newInstance(), i));         
      }

      return instances;
   }   

   public CIMInstance getPowerSupply(RaidSystem system, CIMClass cc, CIMObjectPath op) throws CIMException
   {
      try
      {
         String deviceID = (String)getKeyValue(op, "DeviceID");
         int supplyID = Integer.parseInt(getToken(deviceID, ":", "PowerSupplyID")) - 1;
         int enclosureID = Integer.parseInt(getToken(deviceID, ":", "EnclosureID"));
         int channelID = Channel.displayIDToChannelID(getToken(deviceID, ":", "ChannelID"));
         int controllerID = Adapter.displayIDToAdapterID(getToken(deviceID, ":", "ControllerID"));

         Adapter adapter = (Adapter)system.getAdapter(controllerID);
         
         PhysicalDevice_DeviceIDFilter filter = new PhysicalDevice_DeviceIDFilter(new DeviceID(controllerID, channelID, enclosureID));
         Vector devices = adapter.getPhysicalDeviceCollection(filter);
         EnclosureDevice enclosure = (EnclosureDevice) devices.elementAt(0);
         return enclosure.createCIMPowerSupplyInstance(cc.newInstance(),supplyID);
      }
      catch (ClassCastException e)
      {
         e.printStackTrace(log);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }
      catch (NumberFormatException e)
      {
         e.printStackTrace(log);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }      
      catch (NullPointerException e)
      {
         e.printStackTrace(log);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }
      catch (ArrayIndexOutOfBoundsException e)
      {
         e.printStackTrace(log);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }
   }

   // **********************
   // Enclosure Temperature
   // **********************
   public Vector enumTemperatures(RaidSystem system, CIMClass cc) throws CIMException
   {
      Vector instances = new Vector();

      for (Enumeration e = system.getPhysicalDeviceCollection(new PhysicalDeviceTypeFilter(Constants.ENCLOSURE)).elements(); e.hasMoreElements();)
      {
         EnclosureDevice enclosure = (EnclosureDevice)e.nextElement();         
         instances.addElement(enclosure.createCIMTemperatureInstance(cc.newInstance()));
      }

      return instances;
   }   

   public CIMInstance getTemperature(RaidSystem system, CIMClass cc, CIMObjectPath op) throws CIMException
   {
      try
      {
         String deviceID = (String)getKeyValue(op, "DeviceID");
         int tempID = Integer.parseInt(getToken(deviceID, ":", "TemperatureID")) - 1;
         int enclosureID = Integer.parseInt(getToken(deviceID, ":", "EnclosureID"));
         int channelID = Channel.displayIDToChannelID(getToken(deviceID, ":", "ChannelID"));
         int controllerID = Adapter.displayIDToAdapterID(getToken(deviceID, ":", "ControllerID"));

         Adapter adapter = (Adapter)system.getAdapter(controllerID);
         
         PhysicalDevice_DeviceIDFilter filter = new PhysicalDevice_DeviceIDFilter(new DeviceID(controllerID, channelID, enclosureID));
         Vector devices = adapter.getPhysicalDeviceCollection(filter);
         EnclosureDevice enclosure = (EnclosureDevice) devices.elementAt(0);
         return enclosure.createCIMTemperatureInstance(cc.newInstance());
      }
      catch (ClassCastException e)
      {
         e.printStackTrace(log);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }
      catch (NumberFormatException e)
      {
         e.printStackTrace(log);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }      
      catch (NullPointerException e)
      {
         e.printStackTrace(log);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }
      catch (ArrayIndexOutOfBoundsException e)
      {
         e.printStackTrace(log);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }
   }

   // *******************************
   // Enclosure Physical Components
   // *******************************
   public Vector enumSafteChips(RaidSystem system, CIMClass cc) throws CIMException
   {
      Vector instances = new Vector();

      for (Enumeration e = system.getPhysicalDeviceCollection(new PhysicalDeviceTypeFilter(Constants.ENCLOSURE)).elements(); e.hasMoreElements();)
      {
         EnclosureDevice enclosure = (EnclosureDevice)e.nextElement();         
         instances.addElement(enclosure.createCIMSAFTEChipInstance(cc.newInstance()));
      }

      return instances;
   }   

   public CIMInstance getSafteChip(RaidSystem system, CIMClass cc, CIMObjectPath op) throws CIMException
   {
      try
      {
         String tag = (String)getKeyValue(op, "Tag");         

         int controllerID = Adapter.displayIDToAdapterID(getToken(tag, ":", "ControllerID"));
         int channelID = Channel.displayIDToChannelID(getToken(tag, ":", "ChannelID"));
         int deviceID = Integer.parseInt(getToken(tag, ":", "EnclosureID"));

         Adapter adapter = (Adapter)system.getAdapter(controllerID);
         
         PhysicalDevice_DeviceIDFilter filter = new PhysicalDevice_DeviceIDFilter(new DeviceID(controllerID, channelID, deviceID));
         Vector devices = adapter.getPhysicalDeviceCollection(filter);
         EnclosureDevice enclosure = (EnclosureDevice) devices.elementAt(0);
         return enclosure.createCIMSAFTEChipInstance(cc.newInstance());
      }
      catch (ClassCastException e)
      {
         e.printStackTrace(log);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }
      catch (NumberFormatException e)
      {
         e.printStackTrace(log);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }      
      catch (NullPointerException e)
      {
         e.printStackTrace(log);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }
      catch (ArrayIndexOutOfBoundsException e)
      {
         e.printStackTrace(log);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }
   }

    public static void main(String [] args)
    {
        EnclosureProvider provider = new EnclosureProvider();
        Object [] args1 = null;
        try
        {
            RaidSystem system = provider.getRaidSystem();
            System.out.println(system);
            System.out.println("Controller count = " + system.getAdapterCount());
            System.exit(0);
        }
        catch (RemoteException ex)
        {
            System.out.println("Error accessing the ServeRAID Manager agent. Exception: " + ex);
        }
    }
}
