package com.ibm.sysmgt.raidmgr.dataproc.config.iroc;

// ---------------------------------------------------------------------
//  Copyright:    (C) Adaptec Inc., 2002
//
//  Project Des:  Java Common Raid Manager
//
//  Module Name:  IrocBasicArray.java
//
//  Description:
//
//  Revision(s):  Title - Description              Author          Date
//  ------------  ------------------------     --------------   ----------
//  0.0.0         Created for IROC             Clinton Knight    08/05/02

import java.util.*;
import java.io.*;
import com.ibm.sysmgt.raidmgr.util.*;
import com.ibm.sysmgt.raidmgr.common.*;
import com.ibm.sysmgt.raidmgr.dataproc.config.*;
import com.ibm.sysmgt.raidmgr.dataproc.util.*;
import com.ibm.sysmgt.raidmgr.mgtGUI.actions.base.*;
import com.ibm.sysmgt.raidmgr.mgtGUI.actions.raid.*;
import com.sun.wbem.cim.*;

/** This class represents a old-style array of hard drives. */
public class IrocBasicArray extends BasicArray
{
   static final long serialVersionUID = 72100968636568833L;

   private long uniqueID;

   /** Constructor for a BasicArray that IS NOT contained in a spanned array.
     * @param adapterIN  the adapter on which this array is defined
     * @param arrayIDIN  the ID of this array
     * @param sizeIN  the size of this array
     * @param freeSpaceIN  the free space left on this array
     */
   public IrocBasicArray(Adapter adapterIN,
                         int arrayIDIN,
                         int sizeIN,
                         int freeSpaceIN,
                         long uniqueIDIN)
   {
      super(adapterIN, arrayIDIN, sizeIN, freeSpaceIN);

      uniqueID = uniqueIDIN;

      // update containers
      setLogicalDrivesContainer(new IrocLogicalDrives(adapterIN));
      setPhysicalDrivesContainer(new IrocPhysicalDrives(adapterIN));
   }

   /** Constructor for a BasicArray that IS contained in a spanned array.
     * @param adapterIN  the adapter on which this array is defined
     * @param arrayIDIN  the ID of this array
     * @param sizeIN  the size of this array
     * @param freeSpaceIN  the free space left on this array
     * @param spannedArrayIN  the SpannedArray object that contains this array
     */
   public IrocBasicArray( Adapter adapterIN,
                          int arrayIDIN,
                          int sizeIN,
                          int freeSpaceIN,
                          SpannedArray spannedArrayIN)
   {
      super(adapterIN, arrayIDIN, sizeIN, freeSpaceIN, spannedArrayIN);
   }

   /** Returns the uniqueID assigned to this array
     * @return the uniqueID assigned to this array
     */
   public long getUniqueID()
   {
      return uniqueID;
   }

   /** Get the actions (in the form of AbstractRaidAction objects) supported
     * by this RaidObject.  Used for building the contextual Actions menus.
     * @return the Vector of actions
     */
   public Vector getActions()
   {
      Vector actions = new Vector();

      actions.addElement(new HelpNowAction(this));
      actions.addElement(new NullAction());
      actions.addElement(new DeleteArrayAction(this));
      actions.addElement(new IdentifyDriveAction(getAdapter(), IdentifyDriveAction.ID_ARRAY, getID()));

      return actions;
   }

   /** Returns the data used by the display table model.
     * @return  The multidimensional array of value sets used to populate the console display table.
     */
   public Vector getDisplayDataSet()
   {
      Vector v = new Vector();
      Vector vLogicalDrives = getLogicalDriveCollection(new LogicalDriveTypeFilter(LogicalDrive.BASIC));

      v.addElement(new Object[] { new ToolTipString("infoArrayIdentifier"), getDisplayID()});
      v.addElement(new Object[] { new ToolTipString("infoArraySize"), String.valueOf(getSize())});
      v.addElement(new Object[] { new ToolTipString("infoArrayFreeSpace"), String.valueOf(getFreeSpace())});
      v.addElement(new Object[] { new ToolTipString("infoArrayNumLogDrive"), String.valueOf(vLogicalDrives.size())});
      v.addElement(new Object[] { new ToolTipString("infoArrayNumPhysDrive"), String.valueOf(getPhysicalDeviceCollection(null).size())});

      return v;
   }

   /** This method allows providers to display context-sensitive help.
     * Various software components can register their current context
     * with their associated help provider, so that when help is invoked,
     * the correct content is displayed.
     * @return String  the ID of the most contextually-appropriate help content
     */
   public String getHelpContext()
   {
      return "helpIROCBasicArray";
   }

   /** Determine equivalence.
     * @param object  The object to compare to this one
     * @return  True if the objects are functionally equivalent, false otherwise.
     */
   public boolean equals(Object object)
   {
      // Do this first, since it's fast.
      if (this == object)
         return true;

      // We DON'T subclass java.lang.Object, so we must check for superclass equality.
      if (!super.equals(object))
         return false;

      // Finally, we can do the cast and compare relevant fields.
      // nothing to compare

      // Everything matched, so we're equal!
      return true;
   }

   /** Since we override equals(), we must override hashcode().
     * @return a hash value for this object
     */
   public int hashCode()
   {
      int result = 0;
      result ^= super.hashCode();
      return result;
   }

   /** Returns a shallow clone of this object, sans children.
     * @return a shallow clone of this object, sans children
     */
   public RaidObject emptyClone()
   {
      IrocBasicArray array = new IrocBasicArray(getAdapter(), getID(), getSize(), getFreeSpace(), getUniqueID());
      array.removeAll();
      return array;
   }
}
