package com.ibm.sysmgt.raidmgr.dataproc.config.iroc;

// ---------------------------------------------------------------------
//  Copyright:    (C) Adaptec Inc., 2002
//
//  Project Des:  Java Common Raid Manager
//
//  Module Name:  IrocHotSpareDrives.java
//
//  Description:
//
//  Revision(s):  Title - Description              Author          Date
//  ------------  ------------------------     --------------   ----------
//  0.0.0         Created for IROC             Clinton Knight    08/05/02

import java.util.*;
import java.io.*;
import com.ibm.sysmgt.raidmgr.util.*;
import com.ibm.sysmgt.raidmgr.common.*;
import com.ibm.sysmgt.raidmgr.dataproc.util.*;
import com.ibm.sysmgt.raidmgr.dataproc.config.*;
import com.ibm.sysmgt.raidmgr.mgtGUI.actions.base.*;
import com.ibm.sysmgt.raidmgr.mgtGUI.actions.raid.*;

/** This class is the container object at the Adapter level for hot spare HardDrive drive objects.  
  * The only allowable children of HotSpareDrives are HardDrive objects, as enforced 
  * by the add() method in this class.  As a well-behaved RaidObject, this class uses
  * private final fields to enforce identity integrity.
  */
public class IrocHotSpareDrives extends HotSpareDrives
{
   static final long serialVersionUID = 3943923751805844001L;

   /** The IrocHotSpareDrives constructor.
     * @param adapterIN  The Adapter container of this object
     */
   public IrocHotSpareDrives(Adapter adapterIN)
   {
      super(adapterIN);
   }

   /** This method allows providers to display context-sensitive help.
     * Various software components can register their current context
     * with their associated help provider, so that when help is invoked,
     * the correct content is displayed.
     * @return String  the ID of the most contextually-appropriate help content
     */
   public String getHelpContext()
   {
      return "helpHotSpareDrives";
   }

   /** Determine equivalence.
     * @param object  The object to compare to this one
     * @return  True if the objects are functionally equivalent, false otherwise.
     */
   public boolean equals(Object object)
   {
      // Do this first, since it's fast.
      if (this == object)
         return true;

      // We DON'T subclass java.lang.Object, so we must check for superclass equality.
      if (!super.equals(object))
         return false;

      // Finally, we can do the cast and compare relevant fields.
      IrocHotSpareDrives drives = (IrocHotSpareDrives) object;

      // Everything matched, so we're equal!
      return true;
   }

   /** Removes a child RaidObject from this RaidObject.
     * @param object  the RaidObject to remove
     * @return  true if the specified RaidObject was a child of this RaidObject, false otherwise.
     */
   public boolean remove(RaidObject object)
   {
      return super.remove(object);
   }

   /** Since we override equals(), we must override hashcode().
     * @return a hash value for this object
     */
   public int hashCode()
   {
      int result = 0;
      result ^= super.hashCode();
      return result;
   }
}
