package com.ibm.sysmgt.raidmgr.dataproc.config.iroc;

// ---------------------------------------------------------------------
//  Copyright:    (C) Adaptec Inc., 2002
//
//  Project Des:  Java Common Raid Manager
//
//  Module Name:  IrocPhysicalDrives.java
//
//  Description:
//
//  Revision(s):  Title - Description              Author          Date
//  ------------  ------------------------     --------------   ----------
//  0.0.0         Created for IROC             Clinton Knight    08/05/02

import java.util.*;
import java.io.*;
import com.ibm.sysmgt.raidmgr.util.*;
import com.ibm.sysmgt.raidmgr.common.*;
import com.ibm.sysmgt.raidmgr.dataproc.util.*;
import com.ibm.sysmgt.raidmgr.dataproc.config.*;
import com.ibm.sysmgt.raidmgr.mgtGUI.actions.base.*;
import com.ibm.sysmgt.raidmgr.mgtGUI.actions.raid.*;

/** This class is the container object at the Adapter level for physical devices.  
  * The only allowable children of PhysicalDrives are Channel objects, as enforced 
  * by the add() method in this class.  As a well-behaved RaidObject, this class uses
  * private final fields to enforce identity integrity.
  */
public class IrocPhysicalDrives extends PhysicalDrives
{
   static final long serialVersionUID = 1941563682508129703L;

   /** The IrocPhysicalDrives constructor.
     * @param adapterIN  The Adapter container of this object
     */
   public IrocPhysicalDrives(Adapter adapterIN)
   {
      super(adapterIN);
   }

   /** Get the actions (in the form of AbstractRaidAction objects) supported
     * by this RaidObject.  Used for building the contextual Actions menus.
     * @return the Vector of actions
     */
   public Vector getActions()
   {
      RaidObject parent = getParent();

      if (super.getActions() == null)
      {
         Vector actions = new Vector();
         actions.addElement(new HelpNowAction(this));
         actions.addElement(new NullAction());
         
         if (parent instanceof BasicArray)
         {
            actions.addElement(new IdentifyDriveAction(getAdapter(), IdentifyDriveAction.ID_ALL_PHY_DRVS_IN_ARRAY,
                                                       ((BasicArray) parent).getID()));
         }
         else
         {
            actions.addElement(new IdentifyDriveAction(getAdapter(), IdentifyDriveAction.ID_ALL_PHY_DRVS));
         }

         setActions(actions);
      }

      return super.getActions();
   }

   /** This method allows providers to display context-sensitive help.
     * Various software components can register their current context
     * with their associated help provider, so that when help is invoked,
     * the correct content is displayed.
     * @return String  the ID of the most contextually-appropriate help content
     */
   public String getHelpContext()
   {
      RaidObject parent = getParent();

      if (parent instanceof BasicArray)
         return "helpPhysicalDrivesInArray";
      else
         return "helpPhysicalDrives";
   }

   /** Determine equivalence.
     * @param object  The object to compare to this one
     * @return  True if the objects are functionally equivalent, false otherwise.
     */
   public boolean equals(Object object)
   {
      // Do this first, since it's fast.
      if (this == object)
         return true;

      // We DON'T subclass java.lang.Object, so we must check for superclass equality.
      if (!super.equals(object))
         return false;

      // Finally, we can do the cast and compare relevant fields.
      PhysicalDrives drives = (PhysicalDrives) object;

      // Everything matched, so we're equal!
      return true;
   }

   /** Since we override equals(), we must override hashcode().
     * @return a hash value for this object
     */
   public int hashCode()
   {
      int result = 0;
      result ^= super.hashCode();
      return result;
   }
}
