package com.ibm.sysmgt.raidmgr.dataproc.config.iroc;

// ---------------------------------------------------------------------
//  Copyright:    (C) Adaptec Inc., 2002
//
//  Project Des:  Java Common Raid Manager
//
//  Module Name:  SCSIChannel.java
//
//  Description:
//
//  Revision(s):  Title - Description              Author          Date
//  ------------  ------------------------     --------------   ----------
//  0.0.0         Created for IROC             Clinton Knight    08/05/02

import java.util.*;
import java.io.*;
import com.ibm.sysmgt.raidmgr.util.*;
import com.ibm.sysmgt.raidmgr.common.*;
import com.ibm.sysmgt.raidmgr.dataproc.config.*;
import com.ibm.sysmgt.raidmgr.dataproc.util.*;
import com.ibm.sysmgt.raidmgr.mgtGUI.actions.base.*;
import com.ibm.sysmgt.raidmgr.mgtGUI.actions.raid.*;
import com.sun.wbem.cim.*;

/** This class models a SCSI channel on an IrocAdapter. */
public class IrocSCSIChannel extends SCSIChannel
{
   static final long serialVersionUID = -2617278405276523885L;

   /** The SCSIChannel constructor.
     * @param adapterIN  the Adapter object controlling this device
     * @param channelIDIN  the ID of this channel
     * @param initiatorIDIN  the initiator ID set for this channel
     * @param transferSpeedIN  the transfer speed set for this channel
     */
   public IrocSCSIChannel( Adapter adapterIN,
                          int channelIDIN,
                          int initiatorIDIN,
                          int transferSpeedIN)
   {
      super(adapterIN, channelIDIN, initiatorIDIN, transferSpeedIN);
   }

   /** Get the actions (in the form of AbstractRaidAction objects) supported
     * by this RaidObject.  Used for building the contextual Actions menus.
     * @return the Vector of actions
     */
   public Vector getActions()
   {
      Vector actions = new Vector();

      actions.addElement(new HelpNowAction(this));
      actions.addElement(new NullAction());

      if (getParent().getParent() instanceof Array)
         actions.addElement(new IdentifyDriveAction(getAdapter(), IdentifyDriveAction.ID_SCSI_CHN_IN_ARRAY,
                                                    getID(), ((Array) (getParent().getParent())).getID()));
      else // a normal channel object
         actions.addElement(new IdentifyDriveAction(getAdapter(), IdentifyDriveAction.ID_SCSI_CHN, getID()));

      setActions(actions);

      return actions;
   }

   /** Returns the data used by the display table model.
     * @return  The multidimensional array of value sets used to populate the console display table.
     */
   public Vector getDisplayDataSet()
   {
      Vector v = new Vector();
      int numDrives = getPhysicalDriveCount();

      v.addElement(new Object[] { new ToolTipString("infoChannelDevices"), String.valueOf(numDrives)});
      v.addElement(new Object[] { new ToolTipString("infoChannelTransfer"), SCSIChannel.mapTransferSpeed(getTransferSpeed())});
      //v.addElement(new Object[] { new ToolTipString("infoChannelInitiator"), String.valueOf(getInitiatorID())});

      return v;
   }

   /** This method allows providers to display context-sensitive help.
     * Various software components can register their current context
     * with their associated help provider, so that when help is invoked,
     * the correct content is displayed.
     * @return String  the ID of the most contextually-appropriate help content
     */
   public String getHelpContext()
   {
      return "helpIROCSCSIChannel";
   }

   /** Returns a shallow clone of this object, sans children.
     * @return a shallow clone of this object, sans children
     */
   public RaidObject emptyClone()
   {
      return new IrocSCSIChannel(getAdapter(), getID(), getInitiatorID(), getTransferSpeed());
   }
}
