package com.ibm.sysmgt.raidmgr.dataproc.config.iroc;

// ---------------------------------------------------------------------
//  Copyright:    (C) Adaptec Inc., 2002
//
//  Project Des:  Java Common Raid Manager
//
//  Module Name:  IrocSpannedArray.java
//
//  Description:
//
//  Revision(s):  Title - Description              Author          Date
//  ------------  ------------------------     --------------   ----------
//  0.0.0         Created for IROC             Clinton Knight    08/16/02

import java.util.*;
import java.io.*;
import com.ibm.sysmgt.raidmgr.util.*;
import com.ibm.sysmgt.raidmgr.common.*;
import com.ibm.sysmgt.raidmgr.dataproc.util.*;
import com.ibm.sysmgt.raidmgr.dataproc.config.*;
import com.ibm.sysmgt.raidmgr.mgtGUI.actions.base.*;
import com.ibm.sysmgt.raidmgr.mgtGUI.actions.raid.*;
import com.sun.wbem.cim.*;

public class IrocSpannedArray extends SpannedArray implements Constants
{
   static final long serialVersionUID = 2051727303598648373L;

   private long uniqueID;

   public IrocSpannedArray(Adapter adapterIN, int arrayIDIN, long uniqueIDIN)
   {
      super(adapterIN, arrayIDIN);
      uniqueID = uniqueIDIN;
   }
   
   /** Returns the uniqueID assigned to this array
     * @return the uniqueID assigned to this array
     */
   public long getUniqueID()
   {
      return uniqueID;
   }

   /** Get the actions (in the form of AbstractRaidAction objects) supported
     * by this RaidObject.  Used for building the contextual Actions menus.
     * @return the Vector of actions
     */
   public Vector getActions()
   {
      Vector actions = new Vector();

      actions.addElement(new HelpNowAction(this));
      actions.addElement(new NullAction());
      actions.addElement(new DeleteSpannedArrayAction(this));
      actions.addElement(new IdentifyDriveAction(getAdapter(), IdentifyDriveAction.ID_SPAN_ARRAY, getID()));
      
      return actions;
   }

   /** Returns the data used by the display table model.
     * @return  The multidimensional array of value sets used to populate the console display table.
     */
   public Vector getDisplayDataSet()
   {
      Vector v = new Vector();
      Vector vLogicalDrives = getLogicalDriveCollection(null);

      v.addElement(new Object[] { new ToolTipString("infoArrayIdentifier"), getDisplayID()});
      v.addElement(new Object[] { new ToolTipString("infoSpannedArraySize"), String.valueOf(getSize())});

      if (vLogicalDrives.size() > 0)
         v.addElement(new Object[] { new ToolTipString("infoArrayFreeSpace"), String.valueOf(getFreeSpace())});

      v.addElement(new Object[] { new ToolTipString("infoArrayNumLogDrive"), String.valueOf(vLogicalDrives.size())});
      v.addElement(new Object[] { new ToolTipString("infoArrayNumPhysDrive"), String.valueOf(getPhysicalDeviceCollection(null).size())});

      return v;
   }           

   /** This method allows providers to display context-sensitive help.
     * Various software components can register their current context
     * with their associated help provider, so that when help is invoked,
     * the correct content is displayed.
     * @return String  the ID of the most contextually-appropriate help content
     */
   public String getHelpContext()
   {
      return "helpIROCSpannedArray";
   }

   /** Returns a shallow clone of this object, sans children.
     * @return a shallow clone of this object, sans children
     */
   public RaidObject emptyClone()
   {
      IrocSpannedArray array = new IrocSpannedArray(getAdapter(), getID(), getUniqueID());

      array.remove(getLogicalDrivesContainer());
      array.remove(getArraysContainer()); 
      return array;
   }
}



